@@ -3,7 +3,8 @@ import collections.abc
 import inspect
 from inspect import Parameter
 import sys
-from typing import Any, Dict, Sequence, Set, Union, get_type_hints
+from types import MappingProxyType
+from typing import Any, Collection, Container, Mapping, Sequence, Union, get_type_hints
 try:
     from typing import get_args, get_origin             # type: ignore
 except ImportError:                                     # pragma: no cover
@@ -23,10 +24,10 @@ EMPTY = Parameter.empty
 
 def add_options_from(func,
                      *,
-                     names: Dict[str, Sequence[str]] = None,
-                     include: Set[str] = None,
-                     exclude: Set[str] = None,
-                     custom: Dict[str, Dict[str, Any]] = None):
+                     names: Mapping[str, Sequence[str]] = MappingProxyType({}),
+                     include: Collection[str] = frozenset(),
+                     exclude: Container[str] = frozenset(),
+                     custom: Mapping[str, Mapping[str, Any]] = MappingProxyType({})):
     """Inspect `func` and add corresponding options to the decorated function.
 
     Args:
@@ -48,11 +49,8 @@ def add_options_from(func,
                      If `func` type hints contain standard collections as type hinting generics
                      for Python < 3.9 (e.g. `list[int]`).
     """
-    include = include or set()
-    names = names or {}
-    custom = custom or {}
     try:
-        p_doc = parse_docstring(func.__doc__ or '')
+        p_doc = parse_docstring(func, ignore=exclude)
     except UnsupportedDocstringStyle:
         p_doc = defaultdict(dict)
     try:
@@ -69,7 +67,7 @@ def add_options_from(func,
             raise  # pragma: no cover
         type_hints = {}
     all_parameters = inspect.signature(func).parameters
-    to_be_used = (include or all_parameters.keys()) - (exclude or set())
+    to_be_used = {name for name in (include or all_parameters.keys()) if name not in exclude}
     parameters = [(name, parameter) for name, parameter in all_parameters.items()
                   if name in to_be_used]
 
@@ -84,11 +82,6 @@ def add_options_from(func,
             if not condition:
                 continue
 
-            try:
-                opt_names = names[name]
-            except KeyError:
-                opt_names = f'--{name.replace("_", "-")}',
-
             kwargs = {}
             if 'help' in p_doc[name]:
                 kwargs['help'] = p_doc[name]['help']
@@ -97,20 +90,33 @@ def add_options_from(func,
                 kwargs['default'] = parameter.default
             else:
                 kwargs['required'] = True
+
+            try:
+                kwargs['type'] = custom[name]['type']
+            except KeyError:
                 try:
-                    kwargs['type'] = custom[name]['type']
+                    tp_hint = type_hints[name]
                 except KeyError:
                     try:
-                        tp_hint = type_hints[name]
+                        tp_hint = p_doc[name]['type']
                     except KeyError:
-                        try:
-                            tp_hint = p_doc[name]['type']
-                        except KeyError:
-                            tp_hint = None
+                        tp_hint = None
+                        if parameter.default is EMPTY:
                             warnings.warn(f'No type hint for parameter {name!r}')
-                    if tp_hint is not None:
-                        kwargs.update(_parse_type_hint_into_kwargs(tp_hint))
+                if tp_hint is not None:
+                    kwargs.update(_parse_type_hint_into_kwargs(tp_hint))
+
             kwargs.update(custom.get(name, {}))
+
+            try:
+                opt_names = names[name]
+            except KeyError:
+                opt_name = name.replace("_", "-")
+                if kwargs.get('is_flag', False):
+                    opt_names = [f'--{opt_name}/--no-{opt_name}']
+                else:
+                    opt_names = [f'--{opt_name}']
+
             click.option(*opt_names, **kwargs)(f)
         return f
 
@@ -119,10 +125,12 @@ def add_options_from(func,
 
 def _parse_type_hint_into_kwargs(tp_hint):
     args, origin = get_args(tp_hint), get_origin(tp_hint)
-    if origin in (list, collections.abc.Sequence):
-        return dict(multiple=True, type=args[0])
+    if tp_hint is bool:
+        return dict(is_flag=True, type=bool)
+    elif origin in (list, collections.abc.Sequence):
+        return dict(multiple=True, type=_parse_type_hint_into_kwargs(args[0])['type'])
     elif origin is tuple:
-        return dict(type=args)
+        return dict(type=tuple(_parse_type_hint_into_kwargs(x)['type'] for x in args))
     elif origin is Union:
-        return dict(type=args[0])
+        return _parse_type_hint_into_kwargs(args[0])
     return dict(type=(origin or tp_hint))
